@@ -1,8 +1,9 @@
 ## system-config-printer
 
-## Copyright (C) 2006, 2007, 2008, 2009 Red Hat, Inc.
-## Copyright (C) 2006 Florian Festi <ffesti@redhat.com>
-## Copyright (C) 2006, 2007, 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2006, 2007, 2008, 2009, 2010, 2011 Red Hat, Inc.
+## Authors:
+##  Florian Festi <ffesti@redhat.com>
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -16,10 +17,11 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-import cups, pprint, os, tempfile, re
+import cups, pprint, os, tempfile, re, string
 import locale
+import packagekit.client, packagekit.enums
 from . import _debugprint
 
 class Printer:
@@ -37,7 +39,18 @@ class Printer:
         self.name = name
         self.connection = connection
         self.class_members = []
-        self.update (**kw)
+        have_kw = len (kw) > 0
+        fetch_attrs = True
+        if have_kw:
+            self.update (**kw)
+            if self.is_class:
+                fetch_attrs = True
+            else:
+                fetch_attrs = False
+
+        if fetch_attrs:
+            self.getAttributes ()
+
         self._ppd = None # load on demand
 
     def __del__ (self):
@@ -48,25 +61,25 @@ class Printer:
         return "<cupshelpers.Printer \"%s\">" % self.name
 
     def _expand_flags(self):
+
+        def _ascii_lower(str):
+            return str.translate(string.maketrans(string.ascii_uppercase,
+                                                  string.ascii_lowercase));
+
         prefix = "CUPS_PRINTER_"
         prefix_length = len(prefix)
 
-        # Use the C locale for lower() (trac #151).
-        current_ctype = locale.setlocale (locale.LC_CTYPE)
-        locale.setlocale (locale.LC_CTYPE, "C")
-
         # loop over cups constants
         for name in cups.__dict__:
             if name.startswith(prefix):
-                attr_name = name[prefix_length:].lower()
+                attr_name = \
+                    _ascii_lower(name[prefix_length:])
                 if attr_name in self._flags_blacklist: continue
                 if attr_name == "class": attr_name = "is_class"
                 # set as attribute
                 setattr(self, attr_name,
                         bool(self.type & getattr(cups, name)))
 
-        locale.setlocale (locale.LC_CTYPE, current_ctype)
-
     def update(self, **kw):
         """
         Update object from printer attributes.
@@ -89,6 +102,11 @@ class Printer:
         if self.is_shared is None:
             self.is_shared = not self.not_shared
         del self.not_shared
+        self.class_members = kw.get('member-names', [])
+        if type (self.class_members) != list:
+            self.class_members = [self.class_members]
+        self.class_members.sort ()
+        self.other_attributes = kw
 
     def getAttributes(self):
         """
@@ -130,11 +148,6 @@ class Printer:
                     
                 if attrs.has_key(name+"-supported"):
                     supported = attrs[name+"-supported"]
-                    # Work around pycups bug fixed in 1.9.40:
-                    # finishings-supported should be a list.
-                    if (type (supported) != list and
-                        type (supported) != tuple):
-                        supported = [supported]
                     self.possible_attributes[name] = (value, supported)
             elif (not key.endswith ("-supported") and
                   key != 'job-sheets-default' and
@@ -161,12 +174,6 @@ class Printer:
         elif attrs.has_key('requesting-user-name-denied'):
             self.except_users = attrs['requesting-user-name-denied']
         self.except_users_string = ', '.join(self.except_users)
-        if (attrs.has_key ('device-uri') and
-            attrs['device-uri'].startswith ("smb:")):
-            # Don't update device-uri for smb printers as we need to have
-            # the original from printers.conf, in case it has authentication
-            # details in it.
-            attrs['device-uri'] = self.device_uri
         self.update (**attrs)
 
     def getServer(self):
@@ -327,7 +334,7 @@ class Printer:
         else:
             self.connection.setPrinterUsersAllowed(self.name, except_users)
 
-    def jobsQueued(self, only_tests=False):
+    def jobsQueued(self, only_tests=False, limit=None):
         """
         Find out whether jobs are queued for this printer.
 
@@ -337,7 +344,12 @@ class Printer:
         """
         ret = []
         try:
-            jobs = self.connection.getJobs ()
+            try:
+                r = ['job-id', 'job-printer-uri', 'job-name']
+                jobs = self.connection.getJobs (requested_attributes=r)
+            except TypeError:
+                # requested_attributes requires pycups 1.9.50
+                jobs = self.connection.getJobs ()
         except cups.IPPError:
             return ret
 
@@ -354,15 +366,53 @@ class Printer:
                 (attrs.has_key ('job-name') and
                  attrs['job-name'] == 'Test Page')):
                 ret.append (id)
+
+                if limit != None and len (ret) == limit:
+                    break
         return ret
 
-    def testsQueued(self):
+    def jobsPreserved(self, limit=None):
+        """
+        Find out whether there are preserved jobs for this printer.
+
+        @return: list of job IDs
+        """
+        ret = []
+        try:
+            try:
+                r = ['job-id', 'job-printer-uri', 'job-state']
+                jobs = self.connection.getJobs (which_jobs='completed',
+                                                requested_attributes=r)
+            except TypeError:
+                # requested_attributes requires pycups 1.9.50
+                jobs = self.connection.getJobs (which_jobs='completed')
+        except cups.IPPError:
+            return ret
+
+        for id, attrs in jobs.iteritems():
+            try:
+                uri = attrs['job-printer-uri']
+                uri = uri[uri.rindex ('/') + 1:]
+            except:
+                continue
+            if uri != self.name:
+                continue
+            if (attrs.get ('job-state',
+                           cups.IPP_JOB_PENDING) < cups.IPP_JOB_COMPLETED):
+                continue
+            ret.append (id)
+            if limit != None and len (ret) == limit:
+                break
+
+        return ret
+
+    def testsQueued(self, limit=None):
         """
         Find out whether test jobs are queued for this printer.
 
         @returns: list of job IDs
         """
-        return self.jobsQueued (only_tests=True)
+        return self.jobsQueued (only_tests=True, limit=limit)
 
     def setAsDefault(self):
         """
@@ -373,26 +423,19 @@ class Printer:
         # Also need to check system-wide lpoptions because that's how
         # previous Fedora versions set the default (bug #217395).
         (tmpfd, tmpfname) = tempfile.mkstemp ()
+        os.remove (tmpfname)
         try:
             resource = "/admin/conf/lpoptions"
-
-            try:
-                # Specifying the fd is allowed in pycups >= 1.9.38
-                self.connection.getFile(resource, fd=tmpfd)
-            except TypeError:
-                self.connection.getFile(resource, tmpfname)
+            self.connection.getFile(resource, fd=tmpfd)
         except cups.HTTPError, (s,):
-            try:
-                os.remove (tmpfname)
-            except OSError:
-                pass
-
             if s == cups.HTTP_NOT_FOUND:
                 return False
 
             raise cups.HTTPError (s)
 
-        lines = file (tmpfname).readlines ()
+        f = os.fdopen (tmpfd, 'r+')
+        f.seek (0)
+        lines = f.readlines ()
         changed = False
         i = 0
         for line in lines:
@@ -406,18 +449,15 @@ class Printer:
                 i += 1
 
         if changed:
-            file (tmpfname, 'w').writelines (lines)
+            f.seek (0)
+            f.writelines (lines)
+            f.truncate ()
+            os.lseek (tmpfd, 0, os.SEEK_SET)
             try:
-                self.connection.putFile (resource, tmpfname)
+                self.connection.putFile (resource, fd=tmpfd)
             except cups.HTTPError, (s,):
-                os.remove (tmpfname)
                 return False
 
-        try:
-            os.remove (tmpfname)
-        except OSError:
-            pass
-
         return changed
 
 def getPrinters(connection):
@@ -430,34 +470,12 @@ def getPrinters(connection):
     """
     printers = connection.getPrinters()
     classes = connection.getClasses()
-    printers_conf = None
     for name, printer in printers.iteritems():
         printer = Printer(name, connection, **printer)
         printers[name] = printer
         if classes.has_key(name):
             printer.class_members = classes[name]
             printer.class_members.sort()
-
-        if printer.device_uri.startswith ("smb:"):
-            # smb: URIs may have been sanitized (authentication details
-            # removed), so fetch the actual details from printers.conf.
-            if not printers_conf:
-                printers_conf = _PrintersConf(connection)
-            if printers_conf.device_uris.has_key(name):
-                printer.device_uri = printers_conf.device_uris[name]
-        if not printer.__dict__.has_key ('discovered'):
-            # The CUPS_PRINTER_DISCOVERED flag is new in pycups-1.9.37.
-            printer.discovered = False
-            if printer.device_uri.startswith ("ipp:"):
-                # ipp: Queues can be automatically created as a reaction
-                # to a queue broadcasted by a remote CUPS server. These
-                # queues are not listed in printers,conf. Mark them so
-                # that we can decide which queue entries in the main window
-                # should be editable/deletable and which not
-                if not printers_conf:
-                    printers_conf = _PrintersConf(connection)
-                if not printers_conf.device_uris.has_key(name):
-                    printer.discovered = True
     return printers
 
 def parseDeviceID (id):
@@ -474,7 +492,7 @@ def parseDeviceID (id):
         if piece.find(":") == -1:
             continue
         name, value = piece.split(":",1)
-        id_dict[name] = value
+        id_dict[name.strip ()] = value.strip()
     if id_dict.has_key ("MANUFACTURER"):
         id_dict.setdefault("MFG", id_dict["MANUFACTURER"])
     if id_dict.has_key ("MODEL"):
@@ -483,7 +501,10 @@ def parseDeviceID (id):
         id_dict.setdefault("CMD", id_dict["COMMAND SET"])
     for name in ["MFG", "MDL", "CMD", "CLS", "DES", "SN", "S", "P", "J"]:
         id_dict.setdefault(name, "")
-    id_dict["CMD"] = id_dict["CMD"].split(',') 
+    if id_dict["CMD"] == '':
+        id_dict["CMD"] = []
+    else:
+        id_dict["CMD"] = id_dict["CMD"].split(',') 
     return id_dict
 
 class Device:
@@ -499,10 +520,11 @@ class Device:
         @type kw: dict
         """
         self.uri = uri
-        self.device_class = kw.get('device-class', 'Unknown') # XXX better default
+        self.device_class = kw.get('device-class', '')
         self.info = kw.get('device-info', '')
-        self.make_and_model = kw.get('device-make-and-model', 'Unknown')
+        self.make_and_model = kw.get('device-make-and-model', '')
         self.id = kw.get('device-id', '')
+        self.location = kw.get('device-location', '')
 
         uri_pieces = uri.split(":")
         self.type =  uri_pieces[0]
@@ -512,6 +534,10 @@ class Device:
 
         self.id_dict = parseDeviceID (self.id)
 
+        s = uri.find("serial=")
+        if s != -1 and not self.id_dict.get ('SN',''):
+            self.id_dict['SN'] = uri[s + 7:]
+
     def __repr__ (self):
         return "<cupshelpers.Device \"%s\">" % self.uri
 
@@ -519,6 +545,9 @@ class Device:
         """
         Compare devices by order of preference.
         """
+        if other == None:
+            return -1
+
         if self.is_class != other.is_class:
             if other.is_class:
                 return -1
@@ -538,6 +567,10 @@ class Device:
                 return 1
             if self.type == "hp" or self.type == "hpfax":
                 return -1
+            if other.type == "dnssd":
+                return 1
+            if self.type == "dnssd":
+                return -1
             if other.type == "usb":
                 return 1
             if self.type == "usb":
@@ -548,61 +581,30 @@ class Device:
         
         return result
 
-class _PrintersConf:
-    def __init__(self, connection):
-        """
-        @param connection: CUPS connection
-        @type connection: CUPS.Connection object
-        """
-        self.device_uris = {}
-        self.connection = connection
-        self.parse(self.fetch('/admin/conf/printers.conf'))
+class _GetDevicesCall(object):
+    def call (self, connection, kwds):
+        if kwds.has_key ("reply_handler"):
+            self._client_reply_handler = kwds.get ("reply_handler")
+            kwds["reply_handler"] = self._reply_handler
+            return connection.getDevices (**kwds)
 
-    def fetch(self, file):
-        fd, filename = tempfile.mkstemp("printer.conf")
-        try:
-            try:
-                # Specifying the fd is allowed in pycups >= 1.9.38
-                self.connection.getFile(file, fd=fd)
-            except TypeError:
-                self.connection.getFile(file, filename)
-        except cups.HTTPError, e:
-            os.close(fd)
-            try:
-                os.unlink(filename)
-            except OSError:
-                pass
+        self._client_reply_handler = None
+        result = connection.getDevices (**kwds)
+        return self._reply_handler (connection, result)
 
-            if (e.args[0] == cups.HTTP_UNAUTHORIZED or
-                e.args[0] == cups.HTTP_NOT_FOUND):
-                return []
-            else:
-                raise e
+    def _reply_handler (self, connection, devices):
+        for uri, data in devices.iteritems():
+            device = Device(uri, **data)
+            devices[uri] = device
+            if device.info != '' and device.make_and_model == '':
+                device.make_and_model = device.info
 
-        os.close(fd)
-        lines = open(filename).readlines()
-        os.unlink(filename)
-        return lines
-
-    def parse(self, lines):
-        current_printer = None
-        for line in lines:
-            words = line.split()
-            if len(words) == 0:
-                continue
-            if words[0] == "DeviceURI":
-                if len (words) >= 2:
-                    self.device_uris[current_printer] = words[1]
-                else:
-                    self.device_uris[current_printer] = ''
-            else:
-                match = re.match(r"<(Default)?Printer ([^>]+)>\s*\n", line) 
-                if match:
-                    current_printer = match.group(2)
-                if line.strip().find("</Printer>") != -1:
-                    current_printer = None
-
-def getDevices(connection):
+        if self._client_reply_handler:
+            self._client_reply_handler (connection, devices)
+        else:
+            return devices
+            
+def getDevices(connection, **kw):
     """
     Obtain a list of available CUPS devices.
 
@@ -611,13 +613,8 @@ def getDevices(connection):
     @returns: a list of L{Device} objects
     @raise cups.IPPError: IPP Error
     """
-    devices = connection.getDevices()
-    for uri, data in devices.iteritems():
-        device = Device(uri, **data)
-        devices[uri] = device
-        if device.info != 'Unknown' and device.make_and_model == 'Unknown':
-            device.make_and_model = device.info
-    return devices
+    op = _GetDevicesCall ()
+    return op.call (connection, kw)
 
 def activateNewPrinter(connection, name):
     """
@@ -634,18 +631,7 @@ def activateNewPrinter(connection, name):
     connection.acceptJobs (name)
 
     # Set as the default if there is not already a default printer.
-    default_is_set = False
-    try:
-        if connection.getDefault () != None:
-            default_is_set = True
-    except AttributeError: # getDefault appeared in pycups-1.9.31
-        dests = connection.getDests ()
-        # If a default printer is set it will be available from
-        # key (None,None).
-        if dests.has_key ((None, None)):
-            default_is_set = True
-
-    if not default_is_set:
+    if connection.getDefault () == None:
         connection.setDefault (name)
 
 def copyPPDOptions(ppd1, ppd2):
@@ -702,19 +688,22 @@ def setPPDPageSize(ppd, language):
     except:
         _debugprint ("Failed to set PageSize (%s not available?)" % size)
 
-def missingPackagesAndExecutables(ppd):
+def missingExecutables(ppd):
     """
     Check that all relevant executables for a PPD are installed.
 
     @param ppd: PPD
     @type ppd: cups.PPD object
-    @returns: string list pair, representing missing packages and
-    missing executables
+    @returns: string list, representing missing executables
     """
 
     # First, a local function.  How to check that something exists
     # in a path:
     def pathcheck (name, path="/usr/bin:/bin:/usr/local/bin:/usr/local/bin"):
+        if name == "-":
+            # A filter of "-" means that no filter is required,
+            # i.e. the device accepts the given format as-is.
+            return "builtin"
         # Strip out foomatic '%'-style place-holders.
         p = name.find ('%')
         if p != -1:
@@ -746,9 +735,16 @@ def missingPackagesAndExecutables(ppd):
         _debugprint ("%s: NOT found in %s" % (name,path))
         return None
 
-    pkgs_to_install = []
     exes_to_install = []
 
+    def add_missing (exe):
+        # Strip out foomatic '%'-style place-holders.
+        p = exe.find ('%')
+        if p != -1:
+            exe = exe[:p]
+
+        exes_to_install.append (exe)
+
     # Find a 'FoomaticRIPCommandLine' attribute.
     exe = exepath = None
     attr = ppd.findAttr ('FoomaticRIPCommandLine')
@@ -772,7 +768,8 @@ def missingPackagesAndExecutables(ppd):
                 exe = args[0]
                 exepath = pathcheck (exe)
                 if not exepath:
-                    break
+                    add_missing (exe)
+                    continue
 
                 # Main executable found.  But if it's 'gs',
                 # perhaps there is an IJS server we also need
@@ -786,6 +783,9 @@ def missingPackagesAndExecutables(ppd):
                         if arg.startswith (search):
                             exe = arg[len (search):]
                             exepath = pathcheck (exe)
+                            if not exepath:
+                                add_missing (exe)
+
                             break
 
                         argi += 1
@@ -812,51 +812,41 @@ def missingPackagesAndExecutables(ppd):
                     continue
 
                 exepath = pathcheck (exe,
+                                     "/usr/lib/cups/filter:"
                                      "/usr/local/libexec/cups/filter")
+                if not exepath:
+                    add_missing ("/usr/lib/cups/filter/" + exe)
 
-    if exe and not exepath:
-        # We didn't find a necessary executable.  Complain.
+    return exes_to_install
 
-        # Strip out foomatic '%'-style place-holders.
-        p = exe.find ('%')
-        if p != -1:
-            exe = exe[:p]
+def missingPackagesAndExecutables(ppd):
+    """
+    Check that all relevant executables for a PPD are installed.
 
-        pkgs = {
-            # Foomatic command line executables
-            'gs': 'ghostscript',
-            'perl': 'perl',
-            'foo2oak-wrapper': None,
-            'pnm2ppa': 'pnm2ppa',
-            'c2050': 'c2050',
-            'c2070': 'c2070',
-            'cjet': 'cjet',
-            'lm1100': 'lx',
-            'esc-m': 'min12xxw',
-            'min12xxw': 'min12xxw',
-            'pbm2l2030': 'pbm2l2030',
-            'pbm2l7k': 'pbm2l7k',
-            'pbm2lex': 'pbm2l7k',
-            # IJS servers (used by foomatic)
-            'hpijs': 'hpijs',
-            'ijsgutenprint.5.0': 'gutenprint',
-            # CUPS filters
-            'rastertogutenprint.5.0': 'gutenprint-cups',
-            'commandtoepson': 'gutenprint-cups',
-            'commandtocanon': 'gutenprint-cups',
-            }
-        try:
-            pkg = pkgs[exe]
-        except:
-            pkg = None
+    @param ppd: PPD
+    @type ppd: cups.PPD object
+    @returns: string list pair, representing missing packages and
+    missing executables
+    """
+    executables = missingExecutables(ppd)
+    packages = []
+    if executables:
+        unresolved_executables = []
+        client = packagekit.client.PackageKitClient ()
+        for executable in executables:
+            if not executable.startswith ("/"):
+                executable = "/usr/bin/" + executable
 
-        if pkg:
-            _debugprint ("%s included in package %s" % (exe, pkg))
-            pkgs_to_install.append (pkg)
-        else:
-            exes_to_install.append (exe)
+            result = client.search_file ([executable],
+                                         packagekit.enums.FILTER_NOT_INSTALLED)
+            if result:
+                packages.extend (set (map (lambda x: x.name, result)))
+            else:
+                unresolved_executables.append (executable)
 
-    return (pkgs_to_install, exes_to_install)
+        executables = unresolved_executables
+        
+    return (packages, executables)
 
 def _main():
     c = cups.Connection()
