@@ -16,12 +16,11 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 prefix="/usr/local"
 datadir="/usr/local/share"
 localedir="/usr/local/share/locale"
 pkgdatadir="/usr/local/share/system-config-printer"
-VERSION="1.0.16"
+VERSION="1.3.5"
 PACKAGE="system-config-printer"
-DOWNLOADABLE_DRIVER_SUPPORT=False
