@@ -1,7 +1,8 @@
-#!/usr/local/bin/python2.6
+#!/usr/local/bin/python2.7
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -15,7 +16,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import sys
 import traceback
@@ -24,7 +25,8 @@ _debug=False
 def debugprint (x):
     if _debug:
         try:
-            print >>sys.stderr, x
+            sys.stderr.write (x + "\n")
+            sys.stderr.flush ()
         except:
             pass
 
