@@ -1,9 +1,10 @@
-#!/usr/local/bin/python2.6
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,37 +18,23 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 import os
 import tempfile
 import time
+from timedops import TimedOperation
 from base import *
 class ErrorLogFetch(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Error log fetch")
-        page = self.initial_vbox (_("Debugging"),
-                                  _("I would like to disable debugging output "
-                                    "from the CUPS scheduler.  This may "
-                                    "cause the scheduler to restart.  Click "
-                                    "the button below to disable debugging."))
-        button = gtk.Button (_("Disable Debugging"))
-        buttonbox = gtk.HButtonBox ()
-        buttonbox.set_border_width (0)
-        buttonbox.set_layout (gtk.BUTTONBOX_START)
-        buttonbox.pack_start (button, False, False, 0)
-        self.button = button
-        page.pack_start (buttonbox, False, False, 0)
-        self.label = gtk.Label ()
-        self.label.set_alignment (0, 0)
-        self.label.set_line_wrap (True)
-        page.pack_start (self.label, False, False, 0)
-        troubleshooter.new_page (page, self)
+        troubleshooter.new_page (gtk.Label (), self)
         self.persistent_answers = {}
 
     def display (self):
         answers = self.troubleshooter.answers
+        parent = self.troubleshooter.get_window ()
         self.answers = {}
         try:
             checkpoint = answers['error_log_checkpoint']
@@ -57,8 +44,7 @@ class ErrorLogFetch(Question):
         if self.persistent_answers.has_key ('error_log'):
             checkpoint = None
 
-        if checkpoint != None:
-            c = self.troubleshooter.answers['_authenticated_connection']
+        def fetch_log (c):
             prompt = c._get_prompt_allowed ()
             c._set_prompt_allowed (False)
             c._connect ()
@@ -76,6 +62,52 @@ class ErrorLogFetch(Question):
 
             c._set_prompt_allowed (prompt)
             if success:
+                return tmpfname
+            return None
+
+        self.authconn = self.troubleshooter.answers['_authenticated_connection']
+        if answers.has_key ('error_log_debug_logging_set'):
+            try:
+                self.op = TimedOperation (self.authconn.adminGetServerSettings,
+                                          parent=parent)
+                settings = self.op.run ()
+            except cups.IPPError:
+                return False
+
+            settings[cups.CUPS_SERVER_DEBUG_LOGGING] = '0'
+            orig_settings = answers['cups_server_settings']
+            settings['MaxLogSize'] = orig_settings.get ('MaxLogSize', '2000000')
+            success = False
+            def set_settings (connection, settings):
+                connection.adminSetServerSettings (settings)
+
+                # Now reconnect.
+                attempt = 1
+                while attempt <= 5:
+                    try:
+                        time.sleep (1)
+                        connection._connect ()
+                        break
+                    except RuntimeError:
+                        # Connection failed
+                        attempt += 1
+
+            try:
+
+                self.op = TimedOperation (set_settings,
+                                          (self.authconn, settings),
+                                          parent=parent)
+                self.op.run ()
+                self.persistent_answers['error_log_debug_logging_unset'] = True
+            except cups.IPPError:
+                pass
+
+        if checkpoint != None:
+            self.op = TimedOperation (fetch_log,
+                                      (self.authconn,),
+                                      parent=parent)
+            tmpfname = self.op.run ()
+            if tmpfname != None:
                 f = file (tmpfname)
                 f.seek (checkpoint)
                 lines = f.readlines ()
@@ -83,52 +115,18 @@ class ErrorLogFetch(Question):
                 self.answers = { 'error_log': map (lambda x: x.strip (),
                                                    lines) }
 
-        if answers.has_key ('error_log_debug_logging_set'):
-            self.label.set_text ('')
-            return True
-
         return False
 
-    def connect_signals (self, handler):
-        self.button_sigid = self.button.connect ('clicked', self.button_clicked)
-
-    def disconnect_signals (self):
-        self.button.disconnect (self.button_sigid)
-
     def collect_answer (self):
         answers = self.persistent_answers.copy ()
         answers.update (self.answers)
         return answers
 
-    def button_clicked (self, button):
-        c = self.troubleshooter.answers['_authenticated_connection']
-        try:
-            settings = c.adminGetServerSettings ()
-        except cups.IPPError:
-            return
+    def cancel_operation (self):
+        self.op.cancel ()
 
-        settings[cups.CUPS_SERVER_DEBUG_LOGGING] = '0'
+        # Abandon the CUPS connection and make another.
         answers = self.troubleshooter.answers
-        orig_settings = answers['cups_server_settings']
-        settings['MaxLogSize'] = orig_settings.get ('MaxLogSize', '2000000')
-        success = False
-        try:
-            c.adminSetServerSettings (settings)
-            success = True
-
-            # Now reconnect.
-            attempt = 1
-            while attempt <= 5:
-                try:
-                    time.sleep (1)
-                    c._connect ()
-                    break
-                except RuntimeError:
-                    # Connection failed
-                    attempt += 1
-        except cups.IPPError:
-            pass
-
-        if success:
-            self.persistent_answers['error_log_debug_logging_unset'] = True
-            self.label.set_text (_("Debug logging disabled."))
+        factory = answers['_authenticated_connection_factory']
+        self.authconn = factory.get_connection ()
+        self.answers['_authenticated_connection'] = self.authconn
