@@ -1,9 +1,10 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008, 2009 Red Hat, Inc.
-## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,7 +18,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 import dbus
@@ -26,11 +27,12 @@ import gobject
 import os
 import pango
 import tempfile
+import time
+from timedops import TimedOperation, OperationCanceled
 
 from base import *
 
 import errordialogs
-errordialogs.set_gettext_function (_)
 from errordialogs import *
 
 DBUS_PATH="/com/redhat/PrinterSpooler"
@@ -50,15 +52,18 @@ class PrintTestPage(Question):
         page.set_spacing (12)
         page.set_border_width (12)
 
-        label = gtk.Label ('<span weight="bold" size="larger">' +
-                           _("Test Page") + '</span>\n\n' +
-                           _("Now print a test page.  If you are having "
-                             "problems printing a specific document, print "
-                             "that document now and mark the print job below."))
+        label = gtk.Label ()
         label.set_alignment (0, 0)
         label.set_use_markup (True)
         label.set_line_wrap (True)
         page.pack_start (label, False, False, 0)
+        self.main_label = label
+        self.main_label_text = ('<span weight="bold" size="larger">' +
+                                _("Test Page") + '</span>\n\n' +
+                                _("Now print a test page.  If you are having "
+                                  "problems printing a specific document, "
+                                  "print that document now and mark the print "
+                                  "job below."))
 
         hbox = gtk.HButtonBox ()
         hbox.set_border_width (0)
@@ -120,9 +125,30 @@ class PrintTestPage(Question):
         troubleshooter.new_page (page, self)
 
     def display (self):
-        if not self.troubleshooter.answers.has_key ('cups_queue'):
+        answers = self.troubleshooter.answers
+        if not answers.has_key ('cups_queue'):
             return False
 
+        parent = self.troubleshooter.get_window ()
+        self.authconn = answers['_authenticated_connection']
+        mediatype = None
+        defaults = answers.get ('cups_printer_ppd_defaults', {})
+        for opts in defaults.values ():
+            for opt, value in opts.iteritems ():
+                if opt == "MediaType":
+                    mediatype = value
+                    break
+
+        if mediatype != None:
+            mediatype_string = '\n\n' + (_("Remember to load paper of type "
+                                           "'%s' into the printer first.") %
+                                         mediatype)
+        else:
+            mediatype_string = ""
+
+        label_text = self.main_label_text + mediatype_string
+        self.main_label.set_markup (label_text)
+
         model = gtk.ListStore (gobject.TYPE_BOOLEAN,
                                gobject.TYPE_INT,
                                gobject.TYPE_STRING,
@@ -132,11 +158,31 @@ class PrintTestPage(Question):
         self.job_to_iter = {}
 
         test_jobs = self.persistent_answers.get ('test_page_job_id', [])
-        self.authconn = self.troubleshooter.answers['_authenticated_connection']
-        c = self.authconn
+        def get_jobs ():
+            c = self.authconn
+            try:
+                r = ["job-id",
+                     "job-name",
+                     "job-state",
+                     "job-printer-uri",
+                     "printer-name"]
+                jobs_dict = c.getJobs (which_jobs='not-completed',
+                                       my_jobs=False,
+                                       requested_attributes=r)
+                completed_jobs_dict = c.getJobs (which_jobs='completed',
+                                                 requested_attributes=r)
+            except TypeError:
+                # requested_attributes requires pycups 1.9.50
+                jobs_dict = c.getJobs (which_jobs='not-completed',
+                                       my_jobs=False)
+                completed_jobs_dict = c.getJobs (which_jobs='completed')
+            return (jobs_dict, completed_jobs_dict)
 
-        jobs_dict = c.getJobs (which_jobs='not-completed',
-                               my_jobs=False)
+        self.op = TimedOperation (get_jobs, parent=parent)
+        try:
+            (jobs_dict, completed_jobs_dict) = self.op.run ()
+        except (OperationCanceled, cups.IPPError):
+            return False
 
         # We want to display the jobs in the queue for this printer...
         try:
@@ -156,8 +202,6 @@ class PrintTestPage(Question):
             try:
                 j = jobs_dict[job]
             except KeyError:
-                if completed_jobs_dict == None:
-                    completed_jobs_dict = c.getJobs (which_jobs='completed')
                 try:
                     j = completed_jobs_dict[job]
                 except KeyError:
@@ -179,13 +223,22 @@ class PrintTestPage(Question):
         self.test_sigid = self.test_cell.connect ('toggled',
                                                   self.test_toggled)
 
-        c = self.authconn
-        self.sub_id = c.createSubscription ("/",
-                                            events=["job-created",
-                                                    "job-completed",
-                                                    "job-stopped",
-                                                    "job-progress",
-                                                    "job-state-changed"])
+        def create_subscription ():
+            c = self.authconn
+            sub_id = c.createSubscription ("/",
+                                           events=["job-created",
+                                                   "job-completed",
+                                                   "job-stopped",
+                                                   "job-progress",
+                                                   "job-state-changed"])
+            return sub_id
+
+        parent = self.troubleshooter.get_window ()
+        self.op = TimedOperation (create_subscription, parent=parent)
+        try:
+            self.sub_id = self.op.run ()
+        except (OperationCanceled, cups.IPPError):
+            pass
 
         try:
             bus = dbus.SystemBus ()
@@ -198,7 +251,7 @@ class PrintTestPage(Question):
                                      path=DBUS_PATH,
                                      dbus_interface=DBUS_IFACE)
 
-        self.timer = gobject.timeout_add (1000, self.update_jobs_list)
+        self.timer = gobject.timeout_add_seconds (1, self.update_jobs_list)
 
     def disconnect_signals (self):
         if self.bus:
@@ -209,8 +262,20 @@ class PrintTestPage(Question):
         self.print_button.disconnect (self.print_sigid)
         self.cancel_button.disconnect (self.cancel_sigid)
         self.test_cell.disconnect (self.test_sigid)
-        c = self.authconn
-        c.cancelSubscription (self.sub_id)
+
+        def cancel_subscription (sub_id):
+            c = self.authconn
+            c.cancelSubscription (sub_id)
+
+        parent = self.troubleshooter.get_window ()
+        self.op = TimedOperation (cancel_subscription,
+                                  (self.sub_id,),
+                                  parent=parent)
+        try:
+            self.op.run ()
+        except (OperationCanceled, cups.IPPError):
+            pass
+
         try:
             del self.sub_seq
         except:
@@ -223,36 +288,60 @@ class PrintTestPage(Question):
             return {}
 
         self.answers = self.persistent_answers.copy ()
+        parent = self.troubleshooter.get_window ()
         success = self.yes.get_active ()
         self.answers['test_page_successful'] = success
 
         class collect_jobs:
-            def __init__ (self, model, conn):
+            def __init__ (self, model):
                 self.jobs = []
-                self.cups = conn
-                self.job_attrs = None
                 model.foreach (self.each, None)
 
             def each (self, model, path, iter, user_data):
-                (test, jobid, printer,
-                 doc, status) = model.get (iter, 0, 1, 2, 3, 4)
+                self.jobs.append (model.get (iter, 0, 1, 2, 3, 4))
+
+        model = self.treeview.get_model ()
+        jobs = collect_jobs (model).jobs
+        def collect_attributes (jobs):
+            job_attrs = None
+            c = self.authconn
+            with_attrs = []
+            for (test, jobid, printer, doc, status) in jobs:
                 attrs = None
                 if test:
                     try:
-                        attrs = self.cups.getJobAttributes (jobid)
+                        attrs = c.getJobAttributes (jobid)
                     except AttributeError:
                         # getJobAttributes was introduced in pycups 1.9.35.
-                        if self.job_attrs == None:
-                            self.job_attrs = self.cups.getJobs(which_jobs='all')
+                        if job_attrs == None:
+                            job_attrs = c.getJobs (which_jobs='all')
+
                         attrs = self.job_attrs[jobid]
 
-                self.jobs.append ((test, jobid, printer, doc, status, attrs))
+                with_attrs.append ((test, jobid, printer, doc, status, attrs))
+
+            return with_attrs
+
+        self.op = TimedOperation (collect_attributes,
+                                  (jobs,),
+                                  parent=parent)
+        try:
+            with_attrs = self.op.run ()
+            self.answers['test_page_job_status'] = with_attrs
+        except (OperationCanceled, cups.IPPError):
+            pass
 
-        model = self.treeview.get_model ()
-        self.answers['test_page_job_status'] = collect_jobs (model,
-                                                             self.authconn).jobs
         return self.answers
 
+    def cancel_operation (self):
+        self.op.cancel ()
+
+        # Abandon the CUPS connection and make another.
+        answers = self.troubleshooter.answers
+        factory = answers['_authenticated_connection_factory']
+        self.authconn = factory.get_connection ()
+        self.answers['_authenticated_connection'] = self.authconn
+
     def handle_dbus_signal (self, *args):
         debugprint ("D-Bus signal caught: updating jobs list soon")
         gobject.source_remove (self.timer)
@@ -278,15 +367,17 @@ class PrintTestPage(Question):
         model.set_value (iter, 4, self.STATE[job_dict['job-state']])
 
     def print_clicked (self, widget):
-        self.persistent_answers['test_page_attempted'] = True
+        now = time.time ()
+        tt = time.localtime (now)
+        when = time.strftime ("%d/%b/%Y:%T %z", tt)
+        self.persistent_answers['test_page_attempted'] = when
         answers = self.troubleshooter.answers
-        c = None
-        try:
-            cups.setUser ('')
-            c = cups.Connection ()
-        except RuntimeError:
-            self.persistent_answers['test_page_submit_failure'] = 'connect'
-            return
+        parent = self.troubleshooter.get_window ()
+
+        def print_test_page (*args, **kwargs):
+            factory = answers['_authenticated_connection_factory']
+            c = factory.get_connection ()
+            return c.printTestPage (*args, **kwargs)
 
         tmpfname = None
         mimetypes = [None, 'text/plain']
@@ -294,14 +385,20 @@ class PrintTestPage(Question):
             try:
                 if mimetype == None:
                     # Default test page.
-                    jobid = c.printTestPage (answers['cups_queue'])
+                    self.op = TimedOperation (print_test_page,
+                                              (answers['cups_queue'],),
+                                              parent=parent)
+                    jobid = self.op.run ()
                 elif mimetype == 'text/plain':
                     (tmpfd, tmpfname) = tempfile.mkstemp ()
                     os.write (tmpfd, "This is a test page.\n")
                     os.close (tmpfd)
-                    jobid = c.printTestPage (answers['cups_queue'],
-                                             file=tmpfname,
-                                             format=mimetype)
+                    self.op = TimedOperation (print_test_page,
+                                              (answers['cups_queue'],),
+                                              kwargs={'file': tmpfname,
+                                                      'format': mimetype},
+                                              parent=parent)
+                    jobid = self.op.run ()
                     try:
                         os.unlink (tmpfname)
                     except OSError:
@@ -313,6 +410,12 @@ class PrintTestPage(Question):
                 jobs.append (jobid)
                 self.persistent_answers['test_page_job_id'] = jobs
                 break
+            except OperationCanceled:
+                self.persistent_answers['test_page_submit_failure'] = 'cancel'
+                break
+            except RuntimeError:
+                self.persistent_answers['test_page_submit_failure'] = 'connect'
+                break
             except cups.IPPError, (e, s):
                 if (e == cups.IPP_DOCUMENT_FORMAT and
                     mimetypes.index (mimetype) < (len (mimetypes) - 1)):
@@ -331,14 +434,26 @@ class PrintTestPage(Question):
 
     def cancel_clicked (self, widget):
         self.persistent_answers['test_page_jobs_cancelled'] = True
-        cups.setUser ('')
-        c = cups.Connection ()
+        jobids = []
         for jobid, iter in self.job_to_iter.iteritems ():
-            try:
-                c.cancelJob (jobid)
-            except cups.IPPError, (e, s):
-                if e != cups.IPP_NOT_POSSIBLE:
-                    self.persistent_answers['test_page_cancel_failure'] = (e, s)
+            jobids.append (jobid)
+
+        def cancel_jobs (jobids):
+            c = self.authconn
+            for jobid in jobids:
+                try:
+                    c.cancelJob (jobid)
+                except cups.IPPError, (e, s):
+                    if e != cups.IPP_NOT_POSSIBLE:
+                        self.persistent_answers['test_page_cancel_failure'] = (e, s)
+
+        self.op = TimedOperation (cancel_jobs,
+                                  (jobids,),
+                                  parent=self.troubleshooter.get_window ())
+        try:
+            self.op.run ()
+        except (OperationCanceled, cups.IPPError):
+            pass
 
     def test_toggled (self, cell, path):
         model = self.treeview.get_model ()
@@ -347,12 +462,29 @@ class PrintTestPage(Question):
         model.set_value (iter, 0, not active)
 
     def update_jobs_list (self):
-        c = self.authconn
+        def get_notifications (self):
+            c = self.authconn
+            try:
+                notifications = c.getNotifications ([self.sub_id],
+                                                    [self.sub_seq + 1])
+            except AttributeError:
+                notifications = c.getNotifications ([self.sub_id])
+
+            return notifications
+
+        # Enter the GDK lock.  We need to do this because we were
+        # called from a timeout.
+        gtk.gdk.threads_enter ()
+
+        parent = self.troubleshooter.get_window ()
+        self.op = TimedOperation (get_notifications,
+                                  (self,),
+                                  parent=parent)
         try:
-            notifications = c.getNotifications ([self.sub_id],
-                                                [self.sub_seq + 1])
-        except AttributeError:
-            notifications = c.getNotifications ([self.sub_id])
+            notifications = self.op.run ()
+        except (OperationCanceled, cups.IPPError):
+            gtk.gdk.threads_leave ()
+            return True
 
         answers = self.troubleshooter.answers
         model = self.treeview.get_model ()
@@ -393,9 +525,10 @@ class PrintTestPage(Question):
         # Update again when we're told to. (But we might update sooner if
         # there is a D-Bus signal.)
         gobject.source_remove (self.timer)
-        self.timer = gobject.timeout_add (1000 *
-                                          notifications['notify-get-interval'],
-                                          self.update_jobs_list)
+        self.timer = gobject.timeout_add_seconds (
+            notifications['notify-get-interval'],
+            self.update_jobs_list)
         debugprint ("Update again in %ds" %
                     notifications['notify-get-interval'])
+        gtk.gdk.threads_leave ()
         return False
