@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009 Red Hat, Inc.
+## Copyright (C) 2008, 2009 Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import cups
 from base import *
@@ -60,17 +60,17 @@ class QueueRejectingJobs(Question):
         else:
             reason = queue['printer-state-message']
 
-        text = (_("The queue `%s' is rejecting jobs.") % answers['cups_queue'])
+        text = (_("The queue '%s' is rejecting jobs.") % answers['cups_queue'])
 
         if reason:
-            text += ' ' + _("The reason given is: `%s'.") % reason
+            text += ' ' + _("The reason given is: '%s'.") % reason
 
         if not answers['cups_printer_remote']:
             text += "\n\n"
             text += _("To make the queue accept jobs, select the "
-                      "`Accepting Jobs' checkbox in the `Policies' "
+                      "'Accepting Jobs' checkbox in the 'Policies' "
                       "tab for the printer in the printer administration "
-                      "tool.") + ' ' + TEXT_start_print_admin_tool
+                      "tool.") + ' ' + _(TEXT_start_print_admin_tool)
 
         self.label.set_text (text)
         return True
