@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009, 2010 Red Hat, Inc.
+## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,17 +17,19 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from base import *
 class Shrug(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Shrug")
         page = self.initial_vbox (_("Sorry!"),
-                                  _("I have not been able to work out what "
-                                    "the problem is, but I have collected "
-                                    "some useful information to put in a "
-                                    "bug report."))
+                                  _("There is no obvious solution to this "
+                                    "problem.  Your answers have been "
+                                    "collected together with "
+                                    "other useful information.  If you "
+                                    "would like to report a bug, please "
+                                    "include this information."))
 
         expander = gtk.Expander (_("Diagnostic Output (Advanced)"))
         expander.set_expanded (False)
@@ -45,7 +47,7 @@ class Shrug(Question):
         box.set_layout (gtk.BUTTONBOX_END)
         page.pack_start (box, False, False, 0)
 
-        self.save = gtk.Button (stock='gtk-save')
+        self.save = gtk.Button (stock=gtk.STOCK_SAVE)
         box.pack_start (self.save, False, False, 0)
 
         troubleshooter.new_page (page, self)
@@ -61,15 +63,16 @@ class Shrug(Question):
         self.save.disconnect (self.save_sigid)
 
     def on_save_clicked (self, button):
-        dialog = gtk.FileChooserDialog (parent=self.troubleshooter.main,
+        dialog = gtk.FileChooserDialog (parent=self.troubleshooter.get_window(),
                                         action=gtk.FILE_CHOOSER_ACTION_SAVE,
-                                        buttons=('gtk-cancel',
+                                        buttons=(gtk.STOCK_CANCEL,
                                                  gtk.RESPONSE_CANCEL,
-                                                 'gtk-save',
+                                                 gtk.STOCK_SAVE,
                                                  gtk.RESPONSE_OK))
         dialog.set_do_overwrite_confirmation (True)
         dialog.set_current_name ("troubleshoot.txt")
         dialog.set_default_response (gtk.RESPONSE_OK)
+        dialog.set_local_only (True)
         response = dialog.run ()
         dialog.hide ()
         if response != gtk.RESPONSE_OK:
