@@ -1,9 +1,9 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2009 Red Hat, Inc.
+## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,11 +17,19 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 from base import *
+from timedops import TimedOperation
 import authconn
 
+class AuthConnFactory:
+    def __init__ (self, parent):
+        self.parent = parent
+
+    def get_connection (self):
+        return authconn.Connection (self.parent, lock=True)
+
 class Welcome(Question):
     def __init__ (self, troubleshooter):
         Question.__init__ (self, troubleshooter, "Welcome")
@@ -34,10 +42,10 @@ class Welcome(Question):
         intro = gtk.Label ('<span weight="bold" size="larger">' +
                            _("Trouble-shooting Printing") +
                            '</span>\n\n' +
-                           _("In the next few screens I will ask you some "
+                           _("The next few screens will contain some "
                              "questions about your problem with printing. "
-                             "Based on your answers I will try to suggest "
-                             "a solution.") + '\n\n' +
+                             "Based on your answers a solution may be "
+                             "suggested.") + '\n\n' +
                            _("Click 'Forward' to begin."))
         intro.set_alignment (0, 0)
         intro.set_use_markup (True)
@@ -47,7 +55,13 @@ class Welcome(Question):
         page = troubleshooter.new_page (welcome, self)
 
     def collect_answer (self):
-        parent = self.troubleshooter.main
+        parent = self.troubleshooter.get_window ()
         # Store the authentication dialog instance in the answers.  This
         # allows the password to be cached.
-        return { '_authenticated_connection': authconn.Connection (parent) }
+        factory = AuthConnFactory (parent)
+        self.op = TimedOperation (factory.get_connection, parent=parent)
+        return {'_authenticated_connection_factory': factory,
+                '_authenticated_connection': self.op.run () }
+
+    def cancel_operation (self):
+        self.op.cancel ()
