@@ -1,9 +1,10 @@
-#!/usr/local/bin/python2.5
+#!/usr/local/bin/python2.7
 
 ## Printing troubleshooter
 
-## Copyright (C) 2008 Red Hat, Inc.
-## Copyright (C) 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2008, 2010 Red Hat, Inc.
+## Authors:
+##  Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -17,10 +18,11 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import gtk
 from gettext import gettext as _
+N_ = lambda x: x
 from debug import *
 
 __all__ = [ 'gtk',
@@ -30,9 +32,9 @@ __all__ = [ 'gtk',
             'Multichoice',
             'TEXT_start_print_admin_tool' ]
 
-TEXT_start_print_admin_tool = _("To start this tool, select "
-                                "System->Administration->Printing "
-                                "from the main menu.")
+TEXT_start_print_admin_tool = N_("To start this tool, select "
+                                 "System->Administration->Printing "
+                                 "from the main menu.")
 
 class Question:
     def __init__ (self, troubleshooter, name=None):
@@ -57,6 +59,9 @@ class Question:
     def collect_answer (self):
         return {}
 
+    def cancel_operation (self):
+        pass
+
     ## Helper functions
     def initial_vbox (self, title='', text=''):
         vbox = gtk.VBox ()
