@@ -1,7 +1,7 @@
-#!/usr/local/bin/python2.6
+#!/usr/local/bin/python2.7
 
-## Copyright (C) 2006, 2007, 2008 Red Hat, Inc.
-## Copyright (C) 2007, 2008 Tim Waugh <twaugh@redhat.com>
+## Copyright (C) 2006, 2007, 2008, 2010 Red Hat, Inc.
+## Author: Tim Waugh <twaugh@redhat.com>
 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
 
 ## You should have received a copy of the GNU General Public License
 ## along with this program; if not, write to the Free Software
-## Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+## Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 import gtk
 import os
@@ -72,6 +72,7 @@ class UserDefaultPrinter:
 
     def set (self, default):
         p = subprocess.Popen ([ "lpoptions", "-d", default ],
+                              close_fds=True,
                               stdin=file ("/dev/null"),
                               stdout=file ("/dev/null", "w"),
                               stderr=subprocess.PIPE)
@@ -161,7 +162,11 @@ class UserDefaultPrompt:
                 self.userdef.clear ()
             self.set_default_fn (self.name)
         else:
-            self.userdef.set (self.name)
+            try:
+                self.userdef.set (self.name)
+            except Exception, e:
+                print "Error setting default: %s" % repr (e)
+
             self.refresh_fn ()
 
         dialog.destroy ()
